package pep;

/**This Node subclass defines the operations of Episode nodes.*/
public class Episode extends Node{
 /**error tag*/       private boolean isErrorTagged;
 /**search strength*/ private double searchStrength;

 /**Creates a new Episode.*/
 public Episode(){
  super(RunTime.nResponses);
  threshold = .02;
  isErrorTagged = false;
  searchStrength = 6;
 }

 /**Updates activation for each cycle and seraches responses.*/
 @Override
 public void activate(){
  update();
  if(isErrorTagged) searchStrength = .02;
  if(activation>threshold) respSearch();
 }

 /**Activates Response nodes and weakens connections.*/
 public void respSearch(){
  double temp = (activation-threshold)*searchStrength;
  for(int j=0; j<RunTime.nResponses; j++){
   double output = temp*connections[j];
   RunTime.responses[j].retrieved += output;
   Response.totRetrieved += output;
   connections[j] *= (1-(temp*decayConnect));
  }
 }

 /**Connects Episode to activated Response nodes.*/
 public void connect(){
  for(int j=0; j<RunTime.nResponses; j++){
   connections[j] *= (1-writeConnect);
   connections[j] += RunTime.responses[j].activation*writeConnect;
  }
 }

 /**Tags an Episode as an error.*/
 public void tagError(){
  isErrorTagged = true;
 }

 /**Updates activation for the newest episode.*/
 public void focus(){
  input += .05;
  update();
 }
}