package pep;

/**This superclass defines the basic fields and methods of Nodes*/
public class Node{
 /**rate of decay*/                   public double decay;
 /**incoming activation*/             public double input;
 /**current activation*/              public double activation;
 /**threshold to pass on activation*/ public double threshold;
 /**search strength*/                 public double strength;
 /**rate of connection building*/     public double writeConnect;
 /**rate of decay for connections*/   public double decayConnect;
 /**array of connections*/            public double[] connections;
 /**number of connections*/           public static int nConnections = 0;
 /**target truth value*/              public boolean isTarget;

 /**Creates a new Node.*/
 public Node(){
  decay = .01;
  input = 0;
  activation = 0;
  isTarget = false;
 }

 /**Creates a new Node with connections to other nodes.
  * @param connectNum Number of possible connections to other nodes.*/
 public Node(int connectNum){
  this();
  decayConnect = .0001;
  writeConnect = .01;
  connections = new double[connectNum];
 }

 /**Updates activation and searches memory.*/
 public void activate(){
  update();
  if(activation>threshold) memSearch();
  if(RunTime.isStimuli && !isTarget) memConnect();
 }

/**Uses input to update the activation of the Node.*/
 public void update(){
  input = range(input);
  activation = activation*(1-decay) + input*decay;
  activation = range(activation);
  input = 0;
 }

 /**Ranges activation between 0 and 1.
  * @param in Value to be ranged.
  * @return The newly ranged value.*/
 public double range(double in){
  if(in<0) return 0;
  else if(in>1) return 1;
  else return in;
 }

 /**Ranges activation between 0 and max.
  * @param in Value to be ranged.
  * @param max Maximum value to return.
  * @return The newly ranged value.*/
 public double range(double in,double max){
  if(in<0) return 0;
  else if(in>max) return max;
  else return in;
 }

 /**Activates episodes and weakens connections.*/
 public void memSearch(){
  double temp = (activation-threshold)*strength;
  for(int j=0; j<nConnections; j++){
   RunTime.memory[j].input += temp*connections[j];
   connections[j] *= (1-(temp*decayConnect));
  }
 }

 /**Builds a connection to new episode.*/
 public void memConnect(){
  double temp = range(activation-threshold)*strength;
  RunTime.memory[nConnections].input += temp*connections[nConnections];
  connections[nConnections] *= (1-writeConnect);
  connections[nConnections] += temp*writeConnect;
 }

 /**Sets memory search strength for target.*/
 public void targeted(){
  strength = 0;
  isTarget = true;
 }

 /**Sets memory search strength for distracter.
  * @param nSets The number of distracter sets.*/
 public void ignored(int nSets){
  strength = 6/(double)nSets;
  isTarget = false;
 }
}